﻿// Shawn Poulson, http://explodingcoder.com
// 2010.02.04
//
// Released under Creative Commons Attribution-Share Alike license
// http://creativecommons.org/licenses/by-sa/2.5/

using System;

namespace Expl.Auxiliary {
   /// <summary>
   /// Interface for object-level validation container.
   /// Contains a value that can be validated.
   /// </summary>
   /// <typeparam name="T">Contained value type</typeparam>
   public interface IValidatable<T> {
      /// <summary>
      /// Contained value
      /// </summary>
      T Value { get; set; }

      /// <summary>
      /// Validate this object
      /// </summary>
      /// <returns></returns>
      bool IsValid();
   }

   /// <summary>
   /// Timeout validation container.
   /// Contained value is invalidated when the timeout expires.
   /// </summary>
   /// <typeparam name="T">Contained value type</typeparam>
   public class Timeoutable<T> : IValidatable<T> {
      public T Value { get; set; }
      public DateTime ExpirationTime { get; set; }

      public Timeoutable(T value, DateTime expirationTime) {
         Value = value;
         ExpirationTime = expirationTime;
      }

      public Timeoutable(T value, TimeSpan timeoutDuration) {
         Value = value;
         ExpirationTime = DateTime.Now + timeoutDuration;
      }

      public bool IsValid() {
         return DateTime.Now < ExpirationTime;
      }

      public override int GetHashCode() {
         return Value.GetHashCode() + ExpirationTime.GetHashCode();
      }
   }
}
